@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # SPDX-License-Identifier: Apache-2.0
 # Copyright 2018-2021 Michael R. Crusoe
-"""CWL Expression refactoring tool for CWL v1.0 ."""
+"""CWL Expression refactoring tool for CWL v1.1 ."""
 import copy
 import hashlib
 from collections.abc import Mapping
@@ -22,7 +22,7 @@ from ruamel import yaml
 from schema_salad.sourceline import SourceLine
 from schema_salad.utils import json_dumps
 
-import cwl_utils.parser.cwl_v1_0 as cwl
+import cwl_utils.parser.cwl_v1_1 as cwl
 from cwl_utils.errors import JavascriptException, WorkflowException
 from cwl_utils.expression import do_eval, interpolate
 from cwl_utils.types import CWLObjectType, CWLOutputType
@@ -182,7 +182,7 @@ var runtime=$(runtime);"""
         contents += "\n" + "\n".join(expressionLib)
     contents += (
         """
-var ret = function(){"""
+    var ret = function(){"""
         + escape_expression_field(etool.expression.strip()[2:-1])
         + """}();
 process.stdout.write(JSON.stringify(ret));"""
@@ -235,7 +235,7 @@ def traverse(
                     )
                 )
                 wf_inputs.append(
-                    cwl.InputParameter(
+                    cwl.WorkflowInputParameter(
                         id=inp_id,
                         label=inp.label,
                         secondaryFiles=inp.secondaryFiles,
@@ -323,13 +323,13 @@ def load_step(
 
 def generate_etool_from_expr(
     expr: str,
-    target: Union[cwl.CommandInputParameter, cwl.InputParameter],
+    target: Union[cwl.CommandInputParameter, cwl.WorkflowInputParameter],
     no_inputs: bool = False,
     self_type: Optional[
         Union[
-            cwl.InputParameter,
+            cwl.WorkflowInputParameter,
             cwl.CommandInputParameter,
-            List[Union[cwl.InputParameter, cwl.CommandInputParameter]],
+            List[Union[cwl.WorkflowInputParameter, cwl.CommandInputParameter]],
         ]
     ] = None,  # if the "self" input should be a different type than the "result" output
     extra_processes: Optional[
@@ -345,13 +345,11 @@ def generate_etool_from_expr(
             new_type: Union[
                 List[Union[cwl.ArraySchema, cwl.InputRecordSchema]],
                 Union[cwl.ArraySchema, cwl.InputRecordSchema],
-            ] = [clean_type_ids(t.type) for t in self_type if t.type]
-        elif self_type.type:
-            new_type = clean_type_ids(self_type.type)
+            ] = [clean_type_ids(t.type) for t in self_type]
         else:
-            raise WorkflowException(f"Don't know how to make type from '{self_type}'.")
+            new_type = clean_type_ids(self_type.type)
         inputs.append(
-            cwl.InputParameter(
+            cwl.WorkflowInputParameter(
                 id="self",
                 label=self_type.label if not isinstance(self_type, list) else None,
                 secondaryFiles=self_type.secondaryFiles
@@ -390,7 +388,7 @@ def generate_etool_from_expr(
         expression += "\n  var self=inputs.self;"
     expression += (
         """
-  return {"result": function(){"""
+      return {"result": function(){"""
         + expr[2:-2]
         + """}()};
  }"""
@@ -449,7 +447,7 @@ def replace_expr_with_etool(
     expr: str,
     name: str,
     workflow: cwl.Workflow,
-    target: Union[cwl.CommandInputParameter, cwl.InputParameter],
+    target: Union[cwl.CommandInputParameter, cwl.WorkflowInputParameter],
     source: Optional[Union[str, List[Any]]],
     replace_etool: bool = False,
     extra_process: Optional[
@@ -604,11 +602,11 @@ def param_for_source_id(
     process: Union[cwl.CommandLineTool, cwl.Workflow, cwl.ExpressionTool],
     sourcenames: Union[str, List[str]],
     parent: Optional[cwl.Workflow] = None,
-) -> Union[List[cwl.InputParameter], cwl.InputParameter]:
+) -> Union[List[cwl.WorkflowInputParameter], cwl.WorkflowInputParameter]:
     """Find the process input parameter that matches one of the given sourcenames."""
     if isinstance(sourcenames, str):
         sourcenames = [sourcenames]
-    params: List[cwl.InputParameter] = []
+    params: List[cwl.WorkflowInputParameter] = []
     for sourcename in sourcenames:
         if not isinstance(process, cwl.Workflow):
             for param in process.inputs:
@@ -683,7 +681,9 @@ def process_workflow_inputs_and_outputs(
                     raise_type=WorkflowException,
                 ).makeError(TOPLEVEL_FORMAT_EXPR_ERROR.format(param.id.split("#")[-1]))
             if param.secondaryFiles:
-                if get_expression(param.secondaryFiles, inputs, EMPTY_FILE):
+                if hasattr(param.secondaryFiles, "pattern") and get_expression(
+                    param.secondaryFiles.pattern, inputs, EMPTY_FILE
+                ):
                     raise SourceLine(
                         param.loadingOptions.original_doc,
                         "secondaryFiles",
@@ -691,7 +691,7 @@ def process_workflow_inputs_and_outputs(
                     ).makeError(TOPLEVEL_SF_EXPR_ERROR.format(param.id.split("#")[-1]))
                 elif isinstance(param.secondaryFiles, MutableSequence):
                     for index2, entry in enumerate(param.secondaryFiles):
-                        if get_expression(entry, inputs, EMPTY_FILE):
+                        if get_expression(entry.pattern, inputs, EMPTY_FILE):
                             raise SourceLine(
                                 param.loadingOptions.original_doc,
                                 index2,
@@ -742,7 +742,7 @@ def process_workflow_reqs_and_hints(
                             expression = get_expression(envDef.envValue, inputs, None)
                             if expression:
                                 modified = True
-                                target = cwl.InputParameter(
+                                target = cwl.WorkflowInputParameter(
                                     id=None,
                                     type="string",
                                 )
@@ -773,7 +773,7 @@ def process_workflow_reqs_and_hints(
                         expression = get_expression(this_attr, inputs, None)
                         if expression:
                             modified = True
-                            target = cwl.InputParameter(id=None, type="long")
+                            target = cwl.WorkflowInputParameter(id=None, type="long")
                             etool_id = (
                                 "_expression_workflow_ResourceRequirement_{}".format(
                                     attr
@@ -800,7 +800,7 @@ def process_workflow_reqs_and_hints(
                         expression = get_expression(req.listing, inputs, None)
                         if expression:
                             modified = True
-                            target = cwl.InputParameter(
+                            target = cwl.WorkflowInputParameter(
                                 id=None,
                                 type=cwl.InputArraySchema(
                                     ["File", "Directory"], "array", None, None
@@ -826,7 +826,7 @@ def process_workflow_reqs_and_hints(
                             expression = get_expression(entry, inputs, None)
                             if expression:
                                 modified = True
-                                target = cwl.InputParameter(
+                                target = cwl.WorkflowInputParameter(
                                     id=None,
                                     type=cwl.InputArraySchema(
                                         ["File", "Directory"], "array", None, None
@@ -869,7 +869,7 @@ def process_workflow_reqs_and_hints(
                                                 or expr_result["class"] == "Directory"
                                             )
                                         ):
-                                            target = cwl.InputParameter(
+                                            target = cwl.WorkflowInputParameter(
                                                 id=None,
                                                 type=expr_result["class"],
                                             )
@@ -890,7 +890,7 @@ def process_workflow_reqs_and_hints(
                                                 (etool_id, index)
                                             )
                                         elif isinstance(expr_result, str):
-                                            target = cwl.InputParameter(
+                                            target = cwl.WorkflowInputParameter(
                                                 id=None,
                                                 type=["File"],
                                             )
@@ -932,7 +932,7 @@ def process_workflow_reqs_and_hints(
                                     )
                                     if expression:
                                         modified = True
-                                        target = cwl.InputParameter(
+                                        target = cwl.WorkflowInputParameter(
                                             id=None,
                                             type="string",
                                         )
@@ -1063,7 +1063,7 @@ def process_level_reqs(
                         expression = get_expression(envDef.envValue, inputs, None)
                         if expression:
                             modified = True
-                            target = cwl.InputParameter(id=None, type="string")
+                            target = cwl.WorkflowInputParameter(id=None, type="string")
                             etool_id = "_expression_{}_EnvVarRequirement_{}".format(
                                 step_name, env_index
                             )
@@ -1087,7 +1087,7 @@ def process_level_reqs(
                     expression = get_expression(this_attr, inputs, None)
                     if expression:
                         modified = True
-                        target = cwl.InputParameter(id=None, type="long")
+                        target = cwl.WorkflowInputParameter(id=None, type="long")
                         etool_id = "_expression_{}_ResourceRequirement_{}".format(
                             step_name, attr
                         )
@@ -1114,7 +1114,7 @@ def process_level_reqs(
                         target_type = cwl.InputArraySchema(
                             ["File", "Directory"], "array", None, None
                         )
-                        target = cwl.InputParameter(id=None, type=target_type)
+                        target = cwl.WorkflowInputParameter(id=None, type=target_type)
                         etool_id = "_expression_{}_InitialWorkDirRequirement".format(
                             step_name
                         )
@@ -1150,7 +1150,7 @@ def process_level_reqs(
                             target_type = cwl.InputArraySchema(
                                 ["File", "Directory"], "array", None, None
                             )
-                            target = cwl.InputParameter(
+                            target = cwl.WorkflowInputParameter(
                                 id=None,
                                 type=target_type,
                             )
@@ -1207,7 +1207,7 @@ return result; }"""
                                     else:
                                         new_expression = expression
                                     d_target_type = ["File", "Directory"]
-                                    target = cwl.InputParameter(
+                                    target = cwl.WorkflowInputParameter(
                                         id=None,
                                         type=d_target_type,
                                     )
@@ -1237,7 +1237,7 @@ return result; }"""
                                 )
                                 if expression:
                                     modified = True
-                                    target = cwl.InputParameter(
+                                    target = cwl.WorkflowInputParameter(
                                         id=None,
                                         type="string",
                                     )
@@ -1300,7 +1300,7 @@ def traverse_CommandLineTool(
 ) -> bool:
     """Extract any CWL Expressions within the given CommandLineTool into sibling steps."""
     modified = False
-    # don't modifiy clt, modify step.run
+    # don't modify clt, modify step.run
     target_clt = step.run
     inputs = empty_inputs(clt)
     if not step.id:
@@ -1315,7 +1315,7 @@ def traverse_CommandLineTool(
                     inp_id = f"_arguments_{index}"
                     etool_id = f"_expression_{step_id}{inp_id}"
                     target_type = "Any"
-                    target = cwl.InputParameter(id=None, type=target_type)
+                    target = cwl.WorkflowInputParameter(id=None, type=target_type)
                     replace_step_clt_expr_with_etool(
                         expression, etool_id, parent, target, step, replace_etool
                     )
@@ -1341,7 +1341,7 @@ def traverse_CommandLineTool(
                     inp_id = f"_arguments_{index}"
                     etool_id = f"_expression_{step_id}{inp_id}"
                     target_type = "Any"
-                    target = cwl.InputParameter(id=None, type=target_type)
+                    target = cwl.WorkflowInputParameter(id=None, type=target_type)
                     replace_step_clt_expr_with_etool(
                         expression, etool_id, parent, target, step, replace_etool
                     )
@@ -1367,7 +1367,7 @@ def traverse_CommandLineTool(
                 inp_id = f"_{streamtype}"
                 etool_id = f"_expression_{step_id}{inp_id}"
                 target_type = "string"
-                target = cwl.InputParameter(id=None, type=target_type)
+                target = cwl.WorkflowInputParameter(id=None, type=target_type)
                 replace_step_clt_expr_with_etool(
                     expression, etool_id, parent, target, step, replace_etool
                 )
@@ -1407,7 +1407,7 @@ def traverse_CommandLineTool(
                     inp_id = "_{}_glob".format(outp.id.split("#")[-1])
                     etool_id = f"_expression_{step_id}{inp_id}"
                     glob_target_type = ["string", cwl.ArraySchema("string", "array")]
-                    target = cwl.InputParameter(id=None, type=glob_target_type)
+                    target = cwl.WorkflowInputParameter(id=None, type=glob_target_type)
                     replace_step_clt_expr_with_etool(
                         expression, etool_id, parent, target, step, replace_etool
                     )
@@ -1443,7 +1443,7 @@ def traverse_CommandLineTool(
                     sub_wf_outputs = cltool_step_outputs_to_workflow_outputs(
                         step, etool_id, outp_id
                     )
-                    self_type = cwl.InputParameter(
+                    self_type = cwl.WorkflowInputParameter(
                         id=None,
                         type=cwl.InputArraySchema("File", "array", None, None),
                     )
@@ -1620,7 +1620,7 @@ def replace_step_clt_expr_with_etool(
     expr: str,
     name: str,
     workflow: cwl.Workflow,
-    target: cwl.InputParameter,
+    target: cwl.WorkflowInputParameter,
     step: cwl.WorkflowStep,
     replace_etool: bool,
     self_name: Optional[str] = None,
@@ -1655,7 +1655,7 @@ def replace_clt_hintreq_expr_with_etool(
     expr: str,
     name: str,
     workflow: cwl.Workflow,
-    target: cwl.InputParameter,
+    target: cwl.WorkflowInputParameter,
     step: cwl.WorkflowStep,
     replace_etool: bool,
     self_name: Optional[str] = None,
@@ -1690,7 +1690,7 @@ def replace_clt_hintreq_expr_with_etool(
 
 def cltool_inputs_to_etool_inputs(
     tool: cwl.CommandLineTool,
-) -> List[cwl.InputParameter]:
+) -> List[cwl.WorkflowInputParameter]:
     """Copy CommandLineTool input objects into the equivalent ExpressionTool input objects."""
     inputs = yaml.comments.CommentedSeq()
     if tool.inputs:
@@ -1698,7 +1698,7 @@ def cltool_inputs_to_etool_inputs(
             clt_inp_id = clt_inp.id.split("#")[-1].split("/")[-1]
             if not clt_inp_id.startswith("_"):
                 inputs.append(
-                    cwl.InputParameter(
+                    cwl.WorkflowInputParameter(
                         id=clt_inp_id,
                         label=clt_inp.label,
                         secondaryFiles=clt_inp.secondaryFiles,
@@ -1720,7 +1720,7 @@ def cltool_step_outputs_to_workflow_outputs(
     """
     Copy CommandLineTool outputs into the equivalent Workflow output parameters.
 
-    Connects the outputSources for each of the new output paramters to the step
+    Connects the outputSources for each of the new output parameters to the step
     they came from.
     """
     outputs = yaml.comments.CommentedSeq()
@@ -1754,8 +1754,8 @@ def cltool_step_outputs_to_workflow_outputs(
 
 def generate_etool_from_expr2(
     expr: str,
-    target: cwl.InputParameter,
-    inputs: Sequence[Union[cwl.InputParameter, cwl.CommandInputParameter]],
+    target: Union[cwl.CommandInputParameter, cwl.WorkflowInputParameter],
+    inputs: Sequence[Union[cwl.WorkflowInputParameter, cwl.CommandInputParameter]],
     self_name: Optional[str] = None,
     process: Optional[Union[cwl.CommandLineTool, cwl.ExpressionTool]] = None,
     extra_processes: Optional[
@@ -1780,7 +1780,7 @@ def generate_etool_from_expr2(
         expression += f"\n  var self=inputs.{self_name};"
     expression += (
         """
-  return {"result": function(){"""
+      return {"result": function(){"""
         + expr[2:-2]
         + """}()};
  }"""
@@ -1871,12 +1871,12 @@ def traverse_step(
                     raise WorkflowException("target not found")
                 input_source_id = None
                 source_type: Optional[
-                    Union[List[cwl.InputParameter], cwl.InputParameter]
+                    Union[List[cwl.WorkflowInputParameter], cwl.WorkflowInputParameter]
                 ] = None
                 if inp.source:
                     if isinstance(inp.source, MutableSequence):
                         input_source_id = []
-                        source_types: List[cwl.InputParameter] = []
+                        source_types: List[cwl.WorkflowInputParameter] = []
                         for source in inp.source:
                             source_id = source.split("#")[-1]
                             input_source_id.append(source_id)
@@ -1888,7 +1888,7 @@ def traverse_step(
                             else:
                                 if temp_type not in source_types:
                                     source_types.append(temp_type)
-                        source_type = cwl.InputParameter(
+                        source_type = cwl.WorkflowInputParameter(
                             id=None,
                             type=cwl.ArraySchema(source_types, "array"),
                         )
@@ -1955,10 +1955,10 @@ def traverse_step(
     return modified
 
 
-def workflow_step_to_InputParameters(
+def workflow_step_to_WorkflowInputParameters(
     step_ins: List[cwl.WorkflowStepInput], parent: cwl.Workflow, except_in_id: str
-) -> List[cwl.InputParameter]:
-    """Create InputParametes to match the given WorkflowStep inputs."""
+) -> List[cwl.WorkflowInputParameter]:
+    """Create WorkflowInputParametes to match the given WorkflowStep inputs."""
     params = []
     for inp in step_ins:
         if not inp.id:
@@ -1968,18 +1968,10 @@ def workflow_step_to_InputParameters(
             param = copy.deepcopy(param_for_source_id(parent, sourcenames=inp.source))
             if isinstance(param, list):
                 for p in param:
-                    if not p.type:
-                        raise WorkflowException(
-                            f"Don't know how to get type id for '{p}'."
-                        )
                     p.id = inp_id
                     p.type = clean_type_ids(p.type)
                     params.append(p)
             else:
-                if not param.type:
-                    raise WorkflowException(
-                        f"Don't know how to get type id for '{param}'."
-                    )
                 param.id = inp_id
                 param.type = clean_type_ids(param.type)
                 params.append(param)
@@ -1990,24 +1982,26 @@ def replace_step_valueFrom_expr_with_etool(
     expr: str,
     name: str,
     workflow: cwl.Workflow,
-    target: Union[cwl.CommandInputParameter, cwl.InputParameter],
+    target: Union[cwl.CommandInputParameter, cwl.WorkflowInputParameter],
     step: cwl.WorkflowStep,
     step_inp: cwl.WorkflowStepInput,
     original_process: Union[cwl.CommandLineTool, cwl.ExpressionTool],
     original_step_ins: List[cwl.WorkflowStepInput],
     source: Union[str, List[str]],
     replace_etool: bool,
-    source_type: Optional[Union[cwl.InputParameter, List[cwl.InputParameter]]] = None,
+    source_type: Optional[
+        Union[cwl.WorkflowInputParameter, List[cwl.WorkflowInputParameter]]
+    ] = None,
 ) -> None:
     """Replace a WorkflowStep level 'valueFrom' expression with a sibling ExpressionTool step."""
     if not step_inp.id:
         raise WorkflowException(f"Missing id in {step_inp}.")
     step_inp_id = step_inp.id.split("/")[-1]
-    etool_inputs = workflow_step_to_InputParameters(
+    etool_inputs = workflow_step_to_WorkflowInputParameters(
         original_step_ins, workflow, step_inp_id
     )
     if source:
-        source_param = cwl.InputParameter(id="self", type="Any")
+        source_param = cwl.WorkflowInputParameter(id="self", type="Any")
         # TODO: would be nicer to derive a proper type; but in the face of linkMerge, this is easier for now
         etool_inputs.append(source_param)
     temp_etool = generate_etool_from_expr2(
@@ -2031,6 +2025,8 @@ def replace_step_valueFrom_expr_with_etool(
     if source:
         wf_step_inputs.append(cwl.WorkflowStepInput(id="self", source=step_inp.source))
     for wf_step_input in wf_step_inputs:
+        if not wf_step_input.id:
+            continue
         wf_step_input.id = wf_step_input.id.split("/")[-1]
         if wf_step_input.valueFrom:
             wf_step_input.valueFrom = None
